/* callbacks.c                                           */
/*                                                       */
/* This demonstrates how to create a system modal window.*/
/* Note that this cannot be done with a window.  Instead */
/* we have to fake it with the following arrangement.    */
/*                                                       */
/*                |                |                     */
/*                |                |     |               */
/*                |      |         |     |               */
/*                |      |<========|     |               */
/*                |      |         |<====|               */
/*                |                |     |      USER     */
/*                |<===============|     |               */
/*                |                |     |               */
/*                |                |     |               */
/*                |                |     |               */
/*                |                |                     */
/*                                       ^               */
/*                ^      ^         ^     |               */
/*                |      |         |     |               */
/*           REGION    FAKE     DEVICE INPUT             */
/*         USED FOR   WINDOW'S  REGION REGION            */
/*         BLOCKING   REGION                             */
/*           EVENTS                                      */
/*                                                       */
/* We create two regions.  One for blocking all input    */
/* type events (key, mouse) and one to act as a window.  */
/* To create the regions, we create two PtRegion widgets.*/
/* We dump the contents of a picture module into the     */
/* so-called window region.  Since the blocking region   */
/* is opaque to all key and mouse events, nothing behind */
/* it will get them.  So our fake window region behaves  */
/* like a systemwide modal window.                       */
/*                                                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.14A */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "abimport.h"
#include "proto.h"

static int get_input_group( PhEvent_t const *event );
static PtWidget_t *create_modal_region(void);
static int modal_region_event_cb (PtWidget_t *widget, void *dummy, PtCallbackInfo_t *cbinfo);
static PtWidget_t *create_window_region(void);

PtWidget_t	*modal_region_wgt;	// the region that blocks all user events
PtWidget_t	*window_region_wgt;	// the region used as the window
PhRect_t	window_region_rect;	// the window region's rectangle

/*
 * show_window_cb - Callback that brings up our systemwide modal window.
 * In it we create the two regions mentioned in the comments above
 * and we dump the contents of a picture module into one of them.
 * Note that we've deliberately put a PtPane widget into the picture
 * module to act as a background and to indicate how big we want the
 * so-called window region to be.
 */
int
show_window_cb( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
{
	PhArea_t	area;
	PtArg_t		arg;
	PtWidget_t	*wgt;
	
	modal_region_wgt = create_modal_region();
	window_region_wgt = create_window_region();
	// create_window_region() doesn't realize the region widget so its
	// region does not exist yet.
	
	// dump the picture module into the window region widget
	PtSetParentWidget(window_region_wgt);
	ApCreateModule(ABM_sysmodal_pict, NULL, NULL);
	
	area.pos.x = area.pos.y = 20;
	// assuming there's a main container widget in the picture module,
	// get and use its size for the size of the region widget
	wgt = PtWidgetChildBack(window_region_wgt);
	PtWidgetDim(wgt, &area.size);
	PtSetArg(&arg, Pt_ARG_AREA, &area, 0);
	PtSetResources(window_region_wgt, 1, &arg);
	
	PtRealizeWidget(window_region_wgt); // creates the region

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	return( Pt_CONTINUE );
}

/*
 * sysmodal_win_done_cb - Callback attached to a done button in our
 * modal window.  It is used to destroy our modal window and the two
 * regions involved.
 */
int
sysmodal_win_done_cb( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
{
	if (modal_region_wgt != NULL) {
		PtDestroyWidget(modal_region_wgt); // flags as destroyed
		PtSyncWidget(modal_region_wgt); // really destroy it
		modal_region_wgt = NULL;
	}

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	return( Pt_CONTINUE );
}

static int
get_input_group( PhEvent_t const *event )
{
    int         ig;
    const char  *env;

    return
        event != NULL && ( ig = event->input_group ) > 0
    ||  ( env = getenv("PHIG") ) != NULL && ( ig = atoi( env ) ) > 0
    ?   ig : 1;
}

#define MODAL_EVENTS	(Ph_EV_KEY | Ph_EV_PTR_ALL | Ph_EV_BOUNDARY)

/*
 * create_modal_region - This creates the region that will be blocking
 * key and mouse events.
 */
static PtWidget_t *
create_modal_region(void)
{
	int				ig;
    PhArea_t    	area;
    PtArg_t     	arg[10];
	PtWidget_t		*wgt;
    short       	n = 0;
    long        	fields;

	ig = get_input_group(NULL);
    fields = Ph_REGION_PARENT | Ph_REGION_IN_FRONT | Ph_REGION_EV_SENSE |
			 Ph_REGION_EV_OPAQUE | Ph_REGION_INPUT_GROUP;
    PtSetArg(&arg[n++], Pt_ARG_REGION_FIELDS, fields, fields);
    area.pos.x = area.pos.y = 0;
	area.size.w = area.size.h = SHRT_MAX;
	PtSetArg(&arg[n++], Pt_ARG_AREA, &area, 0 );
	PtSetArg(&arg[n++], Pt_ARG_FILL_COLOR, Pg_TRANSPARENT, 0);
    PtSetArg(&arg[n++], Pt_ARG_REGION_PARENT, Ph_ROOT_RID, 0);
    PtSetArg(&arg[n++], Pt_ARG_REGION_INFRONT, Ph_DEV_RID, 0);
    PtSetArg(&arg[n++], Pt_ARG_REGION_SENSE, Ph_EV_KEY, Ph_EV_KEY);
    PtSetArg(&arg[n++], Pt_ARG_REGION_OPAQUE, MODAL_EVENTS, MODAL_EVENTS);
    PtSetArg(&arg[n++], Pt_ARG_REGION_INPUT_GROUP, ig, 0);
    
	PtSetParentWidget(NULL);
    wgt = PtCreateWidget(PtRegion, NULL, n, arg);
	if (wgt) {
	    PtAddEventHandler(wgt, Ph_EV_KEY, modal_region_event_cb, NULL);
	    PtRealizeWidget(wgt); // this creates the actual region
	}
	return wgt;
}

/*
 * modal_region_event_cb - Any key events that hit our blocking region
 * we want to redirect to our modal window.
 */
static int
modal_region_event_cb (PtWidget_t *widget, void *dummy, PtCallbackInfo_t *cbinfo)
{         
	PhEvent_t		*ev = cbinfo->event;
	void			*data;

	/*
	 * take all key events and re-route	them to our window region,
	 * even the ones that don't overlap.  Why don't some overlap?
	 * Because the device region is emitting them at the coordinates
	 * of the mouse.  If the mouse is not on our window region then
	 * we wouldn't get them.  So we make sure they hit our region.
	 * Normally the window manager does this kind of thing for us
	 * but now we are in front of the window manager's region, so
	 * we have to do it ourselves.
	 */

	ev->emitter.rid = ev->collector.rid = PtWidgetRid(window_region_wgt);
	ev->flags = Ph_EVENT_DIRECT;
	ev->num_rects = 1;
	data = PhGetData(ev);
	PhEventEmit(ev, &window_region_rect, data);

	/* eliminate 'unreferenced' warnings */
	widget = widget, dummy = dummy, cbinfo = cbinfo;

	return (Pt_CONTINUE);
}

#define WINDOW_EVENTS	(Ph_EV_KEY | Ph_EV_PTR_ALL | Ph_EV_EXPOSE)

/*
 * create_window_region - This creates our so-called window region.
 * Note that on return from this function, the region has not been
 * created yet, only the PtRegion widget.  This is because it is not
 * until later that we know how big to make it.
 */
static PtWidget_t *
create_window_region(void)
{
	int				ig;
    PtArg_t     	arg[10];
    short       	n = 0;
    long        	fields;

	/*
	 * we make it opaque to exposes so that when the region is destroyed,
	 * an expose will be emitted behind it
	 */

	ig = get_input_group(NULL);
    fields = Ph_REGION_PARENT | Ph_REGION_EV_SENSE |
			 Ph_REGION_EV_OPAQUE | Ph_REGION_INPUT_GROUP;
    PtSetArg(&arg[n++], Pt_ARG_REGION_FIELDS, fields, fields);
	PtSetArg(&arg[n++], Pt_ARG_FILL_COLOR, Pg_TRANSPARENT, 0);
    PtSetArg(&arg[n++], Pt_ARG_REGION_PARENT, PtWidgetRid(modal_region_wgt), 0);
    PtSetArg(&arg[n++], Pt_ARG_REGION_SENSE, WINDOW_EVENTS, WINDOW_EVENTS);
    PtSetArg(&arg[n++], Pt_ARG_REGION_OPAQUE, Ph_EV_EXPOSE, Ph_EV_EXPOSE);
    PtSetArg(&arg[n++], Pt_ARG_REGION_INPUT_GROUP, ig, 0);
    
    return PtCreateWidget(PtRegion, modal_region_wgt, n, arg);
}
